//Datei: DS1821IO.C
// IO-Routinen
// Temperatur messen mit PIC16F84
// und zwei DS1821
//
// Holger Klabunde
// Copyright 2000
// 24.05.2000
// Compiler CC5x

#include "prozess.h"      /* Der verwendete Prozessor */
#include "ds1821.h"
#include "protos.h"

unsigned char mask,count; //fr die Schleifen unten

void PollStatus()
{
 do
  {
   MasterReset(); //Reset senden
   SendByte(READSTATUS); //Status Lesen Kommando senden
   ReadByte(8);          //8 Bit lesen
   if((tempbuf&0x80)>0) return; //Temp gemessen !
  }while(1); //endlos
}

//Sendet die Master Reset Sequenz
//Und fragt Presence ab
void MasterReset()
{
//Die Routinen sind zum Teil doppelt programmiert
//Es gab sonst Probleme beim Timing
 if(mode==1)
  {
   DSINTERN=0; //Fr Master Reset DQ=0;
   Delay100us(6); //Fr min. 480us     

   DSINTERN=1; //Leitung freigeben
   Delay10us(); //Kurz warten

   while(DSINTERN==1) NOP; //Warten bis DS sich meldet
   while(DSINTERN==0) NOP; //Presence Pulse abwarten
  }
 else
  {
   DSEXT_OFF();   //Fr Master Reset DQ=0;
   Delay100us(6); //Fr min. 480us     

   DSEXT_ON();   //Leitung freigeben
   Delay10us();

   while(DSEXT==1) NOP; //Warten bis DS sich meldet
   while(DSEXT==0) NOP; //Presence Pulse abwarten
  } 
   
 Delay100us(6); //Fr min. 480us     
}

void SendByte(unsigned char by)
{
 unsigned char i;
 mask=0x01;

 for(count=0; count<8; count++) //8Bits senden
  {
   if((mask&by)==0) //Ein 0-Bit, min 60us Pause
    {
     if(mode==1)
      {
       DSINTERN=0;
      }
     else
      {
       DSEXT_OFF();
      } 
     
     //Jetzt min 60us warten
     for(i=0; i<9; i++) Delay10us();

    }
   else             //Ein 1-Bit, sehr kurze Pause
    {
     if(mode==1)
      {
       DSINTERN=0;
       NOP;
      }
     else
      {
       DSEXT_OFF();
       NOP; NOP;
      } 
    } 

   //Leitung freigeben
   DSINTERN=1;
   DSEXT_ON();

   Delay100us(1);
   mask<<=1; //Nchstes Bit
  }
}

void ReadByte(unsigned char bi)
{
 unsigned char buf;
 
 mask=0x01;
 buf=0;
 
 for(count=0; count<8; count++) //bi Bits empfangen
  {
   //Flanke fr DS
   if(mode==1)
    {
     DSINTERN=0;
     NOP; NOP;

     //Leitung freigeben
     DSINTERN=1;
     NOP; NOP; NOP; NOP; NOP; NOP; //Kurz warten

     if(DSINTERN==1) buf|=mask;
     else while(DSINTERN==0);  //Lange 0-Phase vom DS abwarten
    }
   else
    {
     DSEXT_OFF();
     NOP;

     //Leitung freigeben
     DSEXT_ON();
     NOP; NOP; NOP; NOP; //Kurz warten

     if(DSEXT==1) buf|=mask;
     else { while(DSEXT==0) NOP; } //Lange 0-Phase vom DS abwarten
    } 

   Delay100us(1);
   
   mask<<=1; //Nchstes Bit
  }

 tempbuf=buf;

 if(bi==9)
  {
   //Flanke fr DS
     if(mode==1)
      {
       DSINTERN=0;
       NOP; NOP;

       //Leitung freigeben
       DSINTERN=1;
       NOP; NOP; NOP; NOP; NOP; NOP; //Kurz warten

       if(DSINTERN==1) tempbuf+=0x100; //9tes Bit lesen
       else { while(DSINTERN==0) NOP; } //Lange 0-Phase vom DS abwarten
      }
     else
      {
       DSEXT_OFF();
       NOP;
       //Leitung freigeben
       DSEXT_ON();
       NOP; NOP; NOP; NOP; //Kurz warten

       if(DSEXT==1) tempbuf+=0x100; //9tes Bit lesen
       else while(DSEXT==0);  //Lange 0-Phase vom DS abwarten
      } 
     
  } //if bi==9

 Delay100us(6);
}